@@ -6,7 +6,7 @@ import (
 	restful "github.com/emicklei/go-restful"
 	"k8s.io/apimachinery/pkg/api/errors"
 
-	v1 "kubevirt.io/client-go/api/v1"
+	v1 "kubevirt.io/api/core/v1"
 	"kubevirt.io/kubevirt/pkg/controller"
 	apimetrics "kubevirt.io/kubevirt/pkg/monitoring/api"
 )
