@@ -21,18 +21,17 @@ package rest
 
 import (
 	"context"
-
 	"crypto/tls"
 	goerror "errors"
 	"fmt"
 	"io"
 	"net/http"
-	"reflect"
 	"strings"
 	"sync"
 
 	"github.com/emicklei/go-restful"
 	v12 "k8s.io/api/core/v1"
+	"k8s.io/apimachinery/pkg/api/equality"
 	"k8s.io/apimachinery/pkg/api/errors"
 	k8smetav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/apimachinery/pkg/fields"
@@ -43,16 +42,30 @@ import (
 
 	"kubevirt.io/kubevirt/pkg/util/status"
 
-	v1 "kubevirt.io/client-go/api/v1"
+	v1 "kubevirt.io/api/core/v1"
 	"kubevirt.io/client-go/kubecli"
 	"kubevirt.io/client-go/log"
 	"kubevirt.io/kubevirt/pkg/controller"
 	virtconfig "kubevirt.io/kubevirt/pkg/virt-config"
 )
 
+const (
+	unmarshalRequestErrFmt       = "Can not unmarshal Request body to struct, error: %s"
+	vmNotRunning                 = "VM is not running"
+	patchingVMFmt                = "Patching VM: %s"
+	jsonpatchTestErr             = "jsonpatch test operation does not apply"
+	patchingVMStatusFmt          = "Patching VM status: %s"
+	vmiNotRunning                = "VMI is not running"
+	vmiGuestAgentErr             = "VMI does not have guest agent connected"
+	prepConnectionErrFmt         = "Cannot prepare connection %s"
+	getRequestErrFmt             = "Cannot GET request %s"
+	defaultProfilerComponentPort = 8443
+)
+
 type SubresourceAPIApp struct {
 	virtCli                 kubecli.KubevirtClient
 	consoleServerPort       int
+	profilerComponentPort   int
 	handlerTLSConfiguration *tls.Config
 	credentialsLock         *sync.Mutex
 	statusUpdater           *status.VMStatusUpdater
@@ -63,6 +76,7 @@ func NewSubresourceAPIApp(virtCli kubecli.KubevirtClient, consoleServerPort int,
 	return &SubresourceAPIApp{
 		virtCli:                 virtCli,
 		consoleServerPort:       consoleServerPort,
+		profilerComponentPort:   defaultProfilerComponentPort,
 		credentialsLock:         &sync.Mutex{},
 		handlerTLSConfiguration: tlsConfiguration,
 		statusUpdater:           status.NewVMStatusUpdater(virtCli),
@@ -75,11 +89,24 @@ type URLResolver func(*v1.VirtualMachineInstance, kubecli.VirtHandlerConn) (stri
 
 func (app *SubresourceAPIApp) prepareConnection(request *restful.Request, validate validation, getVirtHandlerURL URLResolver) (vmi *v1.VirtualMachineInstance, url string, conn kubecli.VirtHandlerConn, statusError *errors.StatusError) {
 
-	var err error
 	vmiName := request.PathParameter("name")
 	namespace := request.PathParameter("namespace")
 
-	vmi, statusError = app.fetchVirtualMachineInstance(vmiName, namespace)
+	vmi, statusError = app.fetchAndValidateVirtualMachineInstance(namespace, vmiName, validate)
+	if statusError != nil {
+		return
+	}
+
+	url, conn, statusError = app.getVirtHandlerFor(vmi, getVirtHandlerURL)
+	if statusError != nil {
+		return
+	}
+
+	return
+}
+
+func (app *SubresourceAPIApp) fetchAndValidateVirtualMachineInstance(namespace, vmiName string, validate validation) (vmi *v1.VirtualMachineInstance, statusError *errors.StatusError) {
+	vmi, statusError = app.FetchVirtualMachineInstance(namespace, vmiName)
 	if statusError != nil {
 		log.Log.Reason(statusError).Errorf("Failed to gather vmi %s in namespace %s.", vmiName, namespace)
 		return
@@ -88,97 +115,34 @@ func (app *SubresourceAPIApp) prepareConnection(request *restful.Request, valida
 	if statusError = validate(vmi); statusError != nil {
 		return
 	}
-
-	if conn, err = app.getVirtHandlerConnForVMI(vmi); err != nil {
-		statusError = errors.NewBadRequest(err.Error())
-		log.Log.Object(vmi).Reason(statusError).Error("Unable to establish connection to virt-handler")
-		return
-	}
-	if url, err = getVirtHandlerURL(vmi, conn); err != nil {
-		statusError = errors.NewBadRequest(err.Error())
-		log.Log.Object(vmi).Reason(statusError).Error("Unable to retrieve target handler URL")
-		return
-	}
-
 	return
 }
 
-func (app *SubresourceAPIApp) streamRequestHandler(request *restful.Request, response *restful.Response, validate validation, getVirtHandlerURL URLResolver) {
-
-	var err error
-	vmi, url, _, statusError := app.prepareConnection(request, validate, getVirtHandlerURL)
-	if statusError != nil {
-		writeError(statusError, response)
-		return
-	}
-
-	upgrader := kubecli.NewUpgrader()
-	clientSocket, err := upgrader.Upgrade(response.ResponseWriter, request.Request, nil)
-	if err != nil {
-		log.Log.Object(vmi).Reason(err).Error("Failed to upgrade client websocket connection")
-		writeError(errors.NewBadRequest(err.Error()), response)
-		return
-	}
-	defer clientSocket.Close()
-
-	conn, _, err := kubecli.Dial(url, app.handlerTLSConfiguration)
-	if err != nil {
-		log.Log.Object(vmi).Reason(err).Error("failed to dial virt-handler for a console connection")
-		writeError(errors.NewInternalError(err), response)
-		return
-	}
-	defer conn.Close()
-
-	copyErr := make(chan error)
-	go func() {
-		_, err := kubecli.Copy(clientSocket, conn)
-		log.Log.Object(vmi).Reason(err).Error("error encountered reading from virt-handler stream")
-		copyErr <- err
-	}()
-
-	go func() {
-		_, err := kubecli.Copy(conn, clientSocket)
-		log.Log.Object(vmi).Reason(err).Error("error encountered reading from client stream")
-		copyErr <- err
-	}()
-
-	// wait for copy to finish and check the result
-	if err = <-copyErr; err != nil && err != io.EOF {
-		log.Log.Object(vmi).Reason(err).Error("Error in websocket proxy")
-		writeError(errors.NewInternalError(err), response)
-		return
-	}
-}
-
-func (app *SubresourceAPIApp) putRequestHandler(request *restful.Request, response *restful.Response, validate validation, getVirtHandlerURL URLResolver) {
-
+func (app *SubresourceAPIApp) putRequestHandler(request *restful.Request, response *restful.Response, validate validation, getVirtHandlerURL URLResolver, dryRun bool) {
 	_, url, conn, statusErr := app.prepareConnection(request, validate, getVirtHandlerURL)
 	if statusErr != nil {
 		writeError(statusErr, response)
 		return
 	}
 
-	err := conn.Put(url, app.handlerTLSConfiguration)
+	if dryRun {
+		return
+	}
+	err := conn.Put(url, app.handlerTLSConfiguration, request.Request.Body)
 	if err != nil {
 		writeError(errors.NewInternalError(err), response)
 		return
 	}
 }
 
-func (app *SubresourceAPIApp) VNCRequestHandler(request *restful.Request, response *restful.Response) {
-	validate := func(vmi *v1.VirtualMachineInstance) *errors.StatusError {
-		// If there are no graphics devices present, we can't proceed
-		if vmi.Spec.Domain.Devices.AutoattachGraphicsDevice != nil && *vmi.Spec.Domain.Devices.AutoattachGraphicsDevice == false {
-			err := fmt.Errorf("No graphics devices are present.")
-			log.Log.Object(vmi).Reason(err).Error("Can't establish VNC connection.")
-			return errors.NewBadRequest(err.Error())
-		}
-		return nil
+// get either the interface with the provided name or the first available interface
+// if no interface is present, return error
+func getTargetInterfaceIP(vmi *v1.VirtualMachineInstance) (string, error) {
+	interfaces := vmi.Status.Interfaces
+	if len(interfaces) < 1 {
+		return "", fmt.Errorf("no network interfaces are present")
 	}
-	getConsoleURL := func(vmi *v1.VirtualMachineInstance, conn kubecli.VirtHandlerConn) (string, error) {
-		return conn.VNCURI(vmi)
-	}
-	app.streamRequestHandler(request, response, validate, getConsoleURL)
+	return interfaces[0].IP, nil
 }
 
 func (app *SubresourceAPIApp) getVirtHandlerConnForVMI(vmi *v1.VirtualMachineInstance) (kubecli.VirtHandlerConn, error) {
@@ -188,29 +152,14 @@ func (app *SubresourceAPIApp) getVirtHandlerConnForVMI(vmi *v1.VirtualMachineIns
 	return kubecli.NewVirtHandlerClient(app.virtCli).Port(app.consoleServerPort).ForNode(vmi.Status.NodeName), nil
 }
 
-func (app *SubresourceAPIApp) ConsoleRequestHandler(request *restful.Request, response *restful.Response) {
-	validate := func(vmi *v1.VirtualMachineInstance) *errors.StatusError {
-		if vmi.Spec.Domain.Devices.AutoattachSerialConsole != nil && *vmi.Spec.Domain.Devices.AutoattachSerialConsole == false {
-			err := fmt.Errorf("No serial consoles are present.")
-			log.Log.Object(vmi).Reason(err).Error("Can't establish a serial console connection.")
-			return errors.NewBadRequest(err.Error())
-		}
-		if vmi.Status.Phase == v1.Failed {
-			return errors.NewConflict(v1.Resource("virtualmachineinstance"), vmi.Name, fmt.Errorf("VMI is in failed status"))
-		}
-		return nil
-	}
-	getConsoleURL := func(vmi *v1.VirtualMachineInstance, conn kubecli.VirtHandlerConn) (string, error) {
-		return conn.ConsoleURI(vmi)
-	}
-	app.streamRequestHandler(request, response, validate, getConsoleURL)
-}
-
 func getChangeRequestJson(vm *v1.VirtualMachine, changes ...v1.VirtualMachineStateChangeRequest) (string, error) {
+
+	var ops []string
+
 	verb := "add"
 	// Special case: if there's no status field at all, add one.
 	newStatus := v1.VirtualMachineStatus{}
-	if reflect.DeepEqual(vm.Status, newStatus) {
+	if equality.Semantic.DeepEqual(vm.Status, newStatus) {
 		for _, change := range changes {
 			newStatus.StateChangeRequests = append(newStatus.StateChangeRequests, change)
 		}
@@ -218,43 +167,50 @@ func getChangeRequestJson(vm *v1.VirtualMachine, changes ...v1.VirtualMachineSta
 		if err != nil {
 			return "", err
 		}
-		update := fmt.Sprintf(`{ "op": "%s", "path": "/status", "value": %s}`, verb, string(statusJson))
+		ops = append(ops, fmt.Sprintf(`{ "op": "%s", "path": "/status", "value": %s}`, verb, string(statusJson)))
+	} else {
 
-		return fmt.Sprintf("[%s]", update), nil
-	}
-
-	failOnConflict := true
-	if len(changes) == 1 && changes[0].Action == v1.StopRequest {
-		// If this is a stopRequest, replace all existing StateChangeRequests.
-		failOnConflict = false
-	}
-
-	if len(vm.Status.StateChangeRequests) != 0 {
-		if failOnConflict {
-			return "", fmt.Errorf("unable to complete request: stop/start already underway")
-		} else {
-			verb = "replace"
+		failOnConflict := true
+		if len(changes) == 1 && changes[0].Action == v1.StopRequest {
+			// If this is a stopRequest, replace all existing StateChangeRequests.
+			failOnConflict = false
 		}
+
+		if len(vm.Status.StateChangeRequests) != 0 {
+			if failOnConflict {
+				return "", fmt.Errorf("unable to complete request: stop/start already underway")
+			} else {
+				verb = "replace"
+			}
+		}
+
+		changeRequests := []v1.VirtualMachineStateChangeRequest{}
+		for _, change := range changes {
+			changeRequests = append(changeRequests, change)
+		}
+
+		oldChangeRequestsJson, err := json.Marshal(vm.Status.StateChangeRequests)
+		if err != nil {
+			return "", err
+		}
+
+		newChangeRequestsJson, err := json.Marshal(changeRequests)
+		if err != nil {
+			return "", err
+		}
+
+		test := fmt.Sprintf(`{ "op": "test", "path": "/status/stateChangeRequests", "value": %s}`, string(oldChangeRequestsJson))
+		update := fmt.Sprintf(`{ "op": "%s", "path": "/status/stateChangeRequests", "value": %s}`, verb, string(newChangeRequestsJson))
+
+		ops = append(ops, test)
+		ops = append(ops, update)
 	}
 
-	changeRequests := []v1.VirtualMachineStateChangeRequest{}
-	for _, change := range changes {
-		changeRequests = append(changeRequests, change)
+	if vm.Status.StartFailure != nil {
+		ops = append(ops, `{ "op": "remove", "path": "/status/startFailure" }`)
 	}
 
-	oldChangeRequestsJson, err := json.Marshal(vm.Status.StateChangeRequests)
-	if err != nil {
-		return "", err
-	}
-
-	newChangeRequestsJson, err := json.Marshal(changeRequests)
-	if err != nil {
-		return "", err
-	}
-
-	test := fmt.Sprintf(`{ "op": "test", "path": "/status/stateChangeRequests", "value": %s}`, string(oldChangeRequestsJson))
-	update := fmt.Sprintf(`{ "op": "%s", "path": "/status/stateChangeRequests", "value": %s}`, verb, string(newChangeRequestsJson))
-	return fmt.Sprintf("[%s, %s]", test, update), nil
+	return string(controller.GeneratePatchBytes(ops)), nil
 }
 
 func getRunningJson(vm *v1.VirtualMachine, running bool) string {
@@ -277,14 +233,31 @@ func (app *SubresourceAPIApp) MigrateVMRequestHandler(request *restful.Request,
 	name := request.PathParameter("name")
 	namespace := request.PathParameter("namespace")
 
+	bodyStruct := &v1.MigrateOptions{}
+	if request.Request.Body != nil {
+		err := yaml.NewYAMLOrJSONDecoder(request.Request.Body, 1024).Decode(&bodyStruct)
+		switch err {
+		case io.EOF, nil:
+			break
+		default:
+			writeError(errors.NewBadRequest(fmt.Sprintf(unmarshalRequestErrFmt, err)), response)
+			return
+		}
+	}
 	vm, err := app.fetchVirtualMachine(name, namespace)
 	if err != nil {
 		writeError(err, response)
 		return
 	}
 
-	if !vm.Status.Ready {
-		writeError(errors.NewConflict(v1.Resource("virtualmachine"), name, fmt.Errorf("VM is not running")), response)
+	vmi, err := app.FetchVirtualMachineInstance(namespace, name)
+	if err != nil {
+		writeError(err, response)
+		return
+	}
+
+	if vmi.Status.Phase != v1.Running {
+		writeError(errors.NewConflict(v1.Resource("virtualmachine"), name, fmt.Errorf(vmNotRunning)), response)
 		return
 	}
 
@@ -303,7 +276,7 @@ func (app *SubresourceAPIApp) MigrateVMRequestHandler(request *restful.Request,
 			Spec: v1.VirtualMachineInstanceMigrationSpec{
 				VMIName: name,
 			},
-		})
+		}, &k8smetav1.CreateOptions{DryRun: bodyStruct.DryRun})
 		if err != nil {
 			return errors.NewInternalError(err)
 		}
@@ -334,7 +307,7 @@ func (app *SubresourceAPIApp) RestartVMRequestHandler(request *restful.Request,
 		case io.EOF, nil:
 			break
 		default:
-			writeError(errors.NewBadRequest(fmt.Sprintf("Can not unmarshal Request body to struct, error: %s", err)), response)
+			writeError(errors.NewBadRequest(fmt.Sprintf(unmarshalRequestErrFmt, err)), response)
 			return
 		}
 	}
@@ -382,10 +355,10 @@ func (app *SubresourceAPIApp) RestartVMRequestHandler(request *restful.Request,
 		return
 	}
 
-	log.Log.Object(vm).V(4).Infof("Patching VM: %s", bodyString)
-	err = app.statusUpdater.PatchStatus(vm, types.JSONPatchType, []byte(bodyString))
+	log.Log.Object(vm).V(4).Infof(patchingVMFmt, bodyString)
+	err = app.statusUpdater.PatchStatus(vm, types.JSONPatchType, []byte(bodyString), &k8smetav1.PatchOptions{DryRun: bodyStruct.DryRun})
 	if err != nil {
-		if strings.Contains(err.Error(), "jsonpatch test operation does not apply") {
+		if strings.Contains(err.Error(), jsonpatchTestErr) {
 			writeError(errors.NewConflict(v1.Resource("virtualmachine"), name, err), response)
 		} else {
 			writeError(errors.NewInternalError(err), response)
@@ -473,7 +446,24 @@ func (app *SubresourceAPIApp) StartVMRequestHandler(request *restful.Request, re
 		return
 	}
 
-	patchType := types.MergePatchType
+	startPaused := false
+	startChangeRequestData := make(map[string]string)
+	bodyStruct := &v1.StartOptions{}
+	if request.Request.Body != nil {
+		err := yaml.NewYAMLOrJSONDecoder(request.Request.Body, 1024).Decode(&bodyStruct)
+		switch err {
+		case io.EOF, nil:
+			break
+		default:
+			writeError(errors.NewBadRequest(fmt.Sprintf(unmarshalRequestErrFmt, err)), response)
+			return
+		}
+		startPaused = bodyStruct.Paused
+	}
+	if startPaused {
+		startChangeRequestData[v1.StartRequestDataPausedKey] = v1.StartRequestDataPausedTrue
+	}
+
 	var patchErr error
 
 	runStrategy, err := vm.RunStrategy()
@@ -481,18 +471,33 @@ func (app *SubresourceAPIApp) StartVMRequestHandler(request *restful.Request, re
 		writeError(errors.NewInternalError(err), response)
 		return
 	}
-	// RunStrategyHalted         -> spec.running = true
+	// RunStrategyHalted         -> spec.running = true / send start request for paused start
 	// RunStrategyManual         -> send start request
 	// RunStrategyAlways         -> doesn't make sense
 	// RunStrategyRerunOnFailure -> doesn't make sense
 	switch runStrategy {
 	case v1.RunStrategyHalted:
-		bodyString := getRunningJson(vm, true)
-		log.Log.Object(vm).V(4).Infof("Patching VM: %s", bodyString)
-		_, patchErr = app.virtCli.VirtualMachine(namespace).Patch(vm.GetName(), patchType, []byte(bodyString))
-	case v1.RunStrategyRerunOnFailure, v1.RunStrategyManual:
-		patchType = types.JSONPatchType
+		pausedStartStrategy := v1.StartStrategyPaused
+		// Send start request if VM should start paused. virt-controller will update RunStrategy upon this request.
+		// No need to send the request if StartStrategy is already set to Paused in VMI Spec.
+		if startPaused && (vm.Spec.Template == nil || vm.Spec.Template.Spec.StartStrategy != &pausedStartStrategy) {
+			patchString, err := getChangeRequestJson(vm, v1.VirtualMachineStateChangeRequest{
+				Action: v1.StartRequest,
+				Data:   startChangeRequestData,
+			})
+			if err != nil {
+				writeError(errors.NewInternalError(err), response)
+				return
+			}
+			log.Log.Object(vm).V(4).Infof(patchingVMStatusFmt, patchString)
+			patchErr = app.statusUpdater.PatchStatus(vm, types.JSONPatchType, []byte(patchString), &k8smetav1.PatchOptions{DryRun: bodyStruct.DryRun})
+		} else {
+			patchString := getRunningJson(vm, true)
+			log.Log.Object(vm).V(4).Infof(patchingVMFmt, patchString)
+			_, patchErr = app.virtCli.VirtualMachine(namespace).Patch(vm.GetName(), types.MergePatchType, []byte(patchString), &k8smetav1.PatchOptions{DryRun: bodyStruct.DryRun})
+		}
 
+	case v1.RunStrategyRerunOnFailure, v1.RunStrategyManual:
 		needsRestart := false
 		if (runStrategy == v1.RunStrategyRerunOnFailure && vmi != nil && vmi.Status.Phase == v1.Succeeded) ||
 			(runStrategy == v1.RunStrategyManual && vmi != nil && vmi.IsFinal()) {
@@ -506,24 +511,24 @@ func (app *SubresourceAPIApp) StartVMRequestHandler(request *restful.Request, re
 		if needsRestart {
 			bodyString, err = getChangeRequestJson(vm,
 				v1.VirtualMachineStateChangeRequest{Action: v1.StopRequest, UID: &vmi.UID},
-				v1.VirtualMachineStateChangeRequest{Action: v1.StartRequest})
+				v1.VirtualMachineStateChangeRequest{Action: v1.StartRequest, Data: startChangeRequestData})
 		} else {
 			bodyString, err = getChangeRequestJson(vm,
-				v1.VirtualMachineStateChangeRequest{Action: v1.StartRequest})
+				v1.VirtualMachineStateChangeRequest{Action: v1.StartRequest, Data: startChangeRequestData})
 		}
 		if err != nil {
 			writeError(errors.NewInternalError(err), response)
 			return
 		}
-		log.Log.Object(vm).V(4).Infof("Patching VM status: %s", bodyString)
-		patchErr = app.statusUpdater.PatchStatus(vm, patchType, []byte(bodyString))
+		log.Log.Object(vm).V(4).Infof(patchingVMStatusFmt, bodyString)
+		patchErr = app.statusUpdater.PatchStatus(vm, types.JSONPatchType, []byte(bodyString), &k8smetav1.PatchOptions{DryRun: bodyStruct.DryRun})
 	case v1.RunStrategyAlways:
 		writeError(errors.NewConflict(v1.Resource("virtualmachine"), name, fmt.Errorf("%v does not support manual start requests", v1.RunStrategyAlways)), response)
 		return
 	}
 
 	if patchErr != nil {
-		if strings.Contains(patchErr.Error(), "jsonpatch test operation does not apply") {
+		if strings.Contains(patchErr.Error(), jsonpatchTestErr) {
 			writeError(errors.NewConflict(v1.Resource("virtualmachine"), name, patchErr), response)
 		} else {
 			writeError(errors.NewInternalError(patchErr), response)
@@ -550,7 +555,7 @@ func (app *SubresourceAPIApp) StopVMRequestHandler(request *restful.Request, res
 		case io.EOF, nil:
 			break
 		default:
-			writeError(errors.NewBadRequest(fmt.Sprintf("Can not unmarshal Request body to struct, error: %s", err)), response)
+			writeError(errors.NewBadRequest(fmt.Sprintf(unmarshalRequestErrFmt, err)), response)
 			return
 		}
 	}
@@ -561,39 +566,27 @@ func (app *SubresourceAPIApp) StopVMRequestHandler(request *restful.Request, res
 		return
 	}
 
-	vmi, err := app.virtCli.VirtualMachineInstance(namespace).Get(name, &k8smetav1.GetOptions{})
-	if err != nil {
-		if !errors.IsNotFound(err) {
-			writeError(errors.NewInternalError(err), response)
-			return
-		}
-
-		writeError(errors.NewConflict(v1.Resource("virtualmachine"), name, fmt.Errorf("VM has no associated VMI running")), response)
-		return
-	}
-
-	if vmi == nil || vmi.Status.Phase == v1.Succeeded || vmi.Status.Phase == v1.Unknown || vmi.Status.Phase == v1.VmPhaseUnset {
-		writeError(errors.NewConflict(v1.Resource("virtualmachine"), name, fmt.Errorf("VM has status %q and is not running", vmi.Status.Phase)), response)
-		return
-	}
-
-	if (vmi.Status.Phase == v1.Failed && bodyStruct.GracePeriod == nil) || (vmi.Status.Phase == v1.Failed && *bodyStruct.GracePeriod != 0) {
-		writeError(errors.NewConflict(v1.Resource("virtualmachine"), name, fmt.Errorf("VM is Failed. Use `--force --grace-period=0` to stop the VM")), response)
-		return
-	}
-
-	patchType := types.MergePatchType
-	var patchErr error
 	runStrategy, err := vm.RunStrategy()
 	if err != nil {
 		writeError(errors.NewInternalError(err), response)
 		return
 	}
 
-	if bodyStruct.GracePeriod != nil {
+	hasVMI := true
+	vmi, err := app.virtCli.VirtualMachineInstance(namespace).Get(name, &k8smetav1.GetOptions{})
+	if err != nil && errors.IsNotFound(err) {
+		hasVMI = false
+	} else if err != nil {
+		writeError(errors.NewInternalError(err), response)
+		return
+	}
+
+	patchType := types.MergePatchType
+	var patchErr error
+	if hasVMI && !vmi.IsFinal() && bodyStruct.GracePeriod != nil {
 		bodyString := getUpdateTerminatingSecondsGracePeriod(*bodyStruct.GracePeriod)
 		log.Log.Object(vmi).V(2).Infof("Patching VMI: %s", bodyString)
-		_, err = app.virtCli.VirtualMachineInstance(namespace).Patch(vmi.GetName(), patchType, []byte(bodyString))
+		_, err = app.virtCli.VirtualMachineInstance(namespace).Patch(vmi.GetName(), patchType, []byte(bodyString), &k8smetav1.PatchOptions{DryRun: bodyStruct.DryRun})
 		if err != nil {
 			writeError(errors.NewInternalError(err), response)
 			return
@@ -602,9 +595,17 @@ func (app *SubresourceAPIApp) StopVMRequestHandler(request *restful.Request, res
 
 	switch runStrategy {
 	case v1.RunStrategyHalted:
+		if !hasVMI || vmi.IsFinal() {
+			writeError(errors.NewConflict(v1.Resource("virtualmachine"), name, fmt.Errorf(vmNotRunning)), response)
+			return
+		}
 		writeError(errors.NewConflict(v1.Resource("virtualmachine"), name, fmt.Errorf("%v does not support manual stop requests", v1.RunStrategyHalted)), response)
 		return
 	case v1.RunStrategyManual:
+		if !hasVMI || vmi.IsFinal() {
+			writeError(errors.NewConflict(v1.Resource("virtualmachine"), name, fmt.Errorf(vmNotRunning)), response)
+			return
+		}
 		// pass the buck and ask virt-controller to stop the VM. this way the
 		// VM will retain RunStrategy = manual
 		patchType = types.JSONPatchType
@@ -614,16 +615,16 @@ func (app *SubresourceAPIApp) StopVMRequestHandler(request *restful.Request, res
 			writeError(errors.NewInternalError(err), response)
 			return
 		}
-		log.Log.Object(vm).V(4).Infof("Patching VM status: %s", bodyString)
-		patchErr = app.statusUpdater.PatchStatus(vm, patchType, []byte(bodyString))
+		log.Log.Object(vm).V(4).Infof(patchingVMStatusFmt, bodyString)
+		patchErr = app.statusUpdater.PatchStatus(vm, patchType, []byte(bodyString), &k8smetav1.PatchOptions{DryRun: bodyStruct.DryRun})
 	case v1.RunStrategyRerunOnFailure, v1.RunStrategyAlways:
 		bodyString := getRunningJson(vm, false)
-		log.Log.Object(vm).V(4).Infof("Patching VM: %s", bodyString)
-		_, patchErr = app.virtCli.VirtualMachine(namespace).Patch(vm.GetName(), patchType, []byte(bodyString))
+		log.Log.Object(vm).V(4).Infof(patchingVMFmt, bodyString)
+		_, patchErr = app.virtCli.VirtualMachine(namespace).Patch(vm.GetName(), patchType, []byte(bodyString), &k8smetav1.PatchOptions{DryRun: bodyStruct.DryRun})
 	}
 
 	if patchErr != nil {
-		if strings.Contains(patchErr.Error(), "jsonpatch test operation does not apply") {
+		if strings.Contains(patchErr.Error(), jsonpatchTestErr) {
 			writeError(errors.NewConflict(v1.Resource("virtualmachine"), name, patchErr), response)
 		} else {
 			writeError(errors.NewInternalError(patchErr), response)
@@ -638,7 +639,7 @@ func (app *SubresourceAPIApp) PauseVMIRequestHandler(request *restful.Request, r
 
 	validate := func(vmi *v1.VirtualMachineInstance) *errors.StatusError {
 		if vmi.Status.Phase != v1.Running {
-			return errors.NewConflict(v1.Resource("virtualmachineinstance"), vmi.Name, fmt.Errorf("VM is not running"))
+			return errors.NewConflict(v1.Resource("virtualmachineinstance"), vmi.Name, fmt.Errorf(vmNotRunning))
 		}
 		if vmi.Spec.LivenessProbe != nil {
 			return errors.NewForbidden(v1.Resource("virtualmachineinstance"), vmi.Name, fmt.Errorf("Pausing VMIs with LivenessProbe is currently not supported"))
@@ -654,7 +655,23 @@ func (app *SubresourceAPIApp) PauseVMIRequestHandler(request *restful.Request, r
 		return conn.PauseURI(vmi)
 	}
 
-	app.putRequestHandler(request, response, validate, getURL)
+	bodyStruct := &v1.PauseOptions{}
+	if request.Request.Body != nil {
+		err := yaml.NewYAMLOrJSONDecoder(request.Request.Body, 1024).Decode(&bodyStruct)
+		switch err {
+		case io.EOF, nil:
+			break
+		default:
+			writeError(errors.NewBadRequest(fmt.Sprintf(unmarshalRequestErrFmt, err)), response)
+			return
+		}
+	}
+	var dryRun bool
+	if len(bodyStruct.DryRun) > 0 && bodyStruct.DryRun[0] == k8smetav1.DryRunAll {
+		dryRun = true
+	}
+	app.putRequestHandler(request, response, validate, getURL, dryRun)
+
 }
 
 func (app *SubresourceAPIApp) UnpauseVMIRequestHandler(request *restful.Request, response *restful.Response) {
@@ -672,8 +689,80 @@ func (app *SubresourceAPIApp) UnpauseVMIRequestHandler(request *restful.Request,
 	getURL := func(vmi *v1.VirtualMachineInstance, conn kubecli.VirtHandlerConn) (string, error) {
 		return conn.UnpauseURI(vmi)
 	}
-	app.putRequestHandler(request, response, validate, getURL)
 
+	bodyStruct := &v1.UnpauseOptions{}
+	if request.Request.Body != nil {
+		err := yaml.NewYAMLOrJSONDecoder(request.Request.Body, 1024).Decode(&bodyStruct)
+		switch err {
+		case io.EOF, nil:
+			break
+		default:
+			writeError(errors.NewBadRequest(fmt.Sprintf(unmarshalRequestErrFmt, err)), response)
+			return
+		}
+	}
+	var dryRun bool
+	if len(bodyStruct.DryRun) > 0 && bodyStruct.DryRun[0] == k8smetav1.DryRunAll {
+		dryRun = true
+	}
+	app.putRequestHandler(request, response, validate, getURL, dryRun)
+
+}
+
+func (app *SubresourceAPIApp) FreezeVMIRequestHandler(request *restful.Request, response *restful.Response) {
+
+	validate := func(vmi *v1.VirtualMachineInstance) *errors.StatusError {
+		if vmi.Status.Phase != v1.Running {
+			return errors.NewConflict(v1.Resource("virtualmachineinstance"), vmi.Name, fmt.Errorf(vmNotRunning))
+		}
+		return nil
+	}
+
+	getURL := func(vmi *v1.VirtualMachineInstance, conn kubecli.VirtHandlerConn) (string, error) {
+		return conn.FreezeURI(vmi)
+	}
+
+	app.putRequestHandler(request, response, validate, getURL, false)
+}
+
+func (app *SubresourceAPIApp) UnfreezeVMIRequestHandler(request *restful.Request, response *restful.Response) {
+
+	validate := func(vmi *v1.VirtualMachineInstance) *errors.StatusError {
+		if vmi.Status.Phase != v1.Running {
+			return errors.NewConflict(v1.Resource("virtualmachineinstance"), vmi.Name, fmt.Errorf(vmiNotRunning))
+		}
+		return nil
+	}
+	getURL := func(vmi *v1.VirtualMachineInstance, conn kubecli.VirtHandlerConn) (string, error) {
+		return conn.UnfreezeURI(vmi)
+	}
+	app.putRequestHandler(request, response, validate, getURL, false)
+
+}
+
+func (app *SubresourceAPIApp) SoftRebootVMIRequestHandler(request *restful.Request, response *restful.Response) {
+
+	validate := func(vmi *v1.VirtualMachineInstance) *errors.StatusError {
+		if vmi.Status.Phase != v1.Running {
+			return errors.NewConflict(v1.Resource("virtualmachineinstance"), vmi.Name, fmt.Errorf(vmNotRunning))
+		}
+		condManager := controller.NewVirtualMachineInstanceConditionManager()
+		if condManager.HasConditionWithStatus(vmi, v1.VirtualMachineInstancePaused, v12.ConditionTrue) {
+			return errors.NewConflict(v1.Resource("virtualmachineinstance"), vmi.Name, fmt.Errorf("VMI is paused"))
+		}
+		if !condManager.HasCondition(vmi, v1.VirtualMachineInstanceAgentConnected) {
+			if features := vmi.Spec.Domain.Features; features != nil && features.ACPI.Enabled != nil && !(*features.ACPI.Enabled) {
+				return errors.NewConflict(v1.Resource("virtualmachineinstance"), vmi.Name, fmt.Errorf("VMI neither have the agent connected nor the ACPI feature enabled"))
+			}
+		}
+		return nil
+	}
+
+	getURL := func(vmi *v1.VirtualMachineInstance, conn kubecli.VirtHandlerConn) (string, error) {
+		return conn.SoftRebootURI(vmi)
+	}
+
+	app.putRequestHandler(request, response, validate, getURL, false)
 }
 
 func (app *SubresourceAPIApp) fetchVirtualMachine(name string, namespace string) (*v1.VirtualMachine, *errors.StatusError) {
@@ -688,7 +777,8 @@ func (app *SubresourceAPIApp) fetchVirtualMachine(name string, namespace string)
 	return vm, nil
 }
 
-func (app *SubresourceAPIApp) fetchVirtualMachineInstance(name string, namespace string) (*v1.VirtualMachineInstance, *errors.StatusError) {
+// FetchVirtualMachineInstance by namespace and name
+func (app *SubresourceAPIApp) FetchVirtualMachineInstance(namespace, name string) (*v1.VirtualMachineInstance, *errors.StatusError) {
 
 	vmi, err := app.virtCli.VirtualMachineInstance(namespace).Get(name, &k8smetav1.GetOptions{})
 	if err != nil {
@@ -700,6 +790,37 @@ func (app *SubresourceAPIApp) fetchVirtualMachineInstance(name string, namespace
 	return vmi, nil
 }
 
+// FetchVirtualMachineInstanceForVM by namespace and name
+func (app *SubresourceAPIApp) FetchVirtualMachineInstanceForVM(namespace, name string) (*v1.VirtualMachineInstance, *errors.StatusError) {
+	vm, err := app.virtCli.VirtualMachine(namespace).Get(name, &k8smetav1.GetOptions{})
+	if err != nil {
+		if errors.IsNotFound(err) {
+			return nil, errors.NewNotFound(v1.Resource("virtualmachine"), name)
+		}
+		return nil, errors.NewInternalError(fmt.Errorf("unable to retrieve vm [%s]: %v", name, err))
+	}
+
+	if !vm.Status.Created {
+		return nil, errors.NewConflict(v1.Resource("virtualmachine"), vm.Name, fmt.Errorf("VMI is not started"))
+	}
+
+	vmi, err := app.virtCli.VirtualMachineInstance(namespace).Get(name, &k8smetav1.GetOptions{})
+	if err != nil {
+		if errors.IsNotFound(err) {
+			return nil, errors.NewNotFound(v1.Resource("virtualmachineinstance"), name)
+		}
+		return nil, errors.NewInternalError(fmt.Errorf("unable to retrieve vmi [%s]: %v", name, err))
+	}
+
+	for _, ref := range vmi.OwnerReferences {
+		if ref.UID == vm.UID {
+			return vmi, nil
+		}
+	}
+
+	return nil, errors.NewInternalError(fmt.Errorf("unable to retrieve vmi [%s] for vm: %v", name, err))
+}
+
 func writeError(error *errors.StatusError, response *restful.Response) {
 	errStatus := error.ErrStatus.DeepCopy()
 	errStatus.Kind = "Status"
@@ -714,11 +835,11 @@ func writeError(error *errors.StatusError, response *restful.Response) {
 func (app *SubresourceAPIApp) GuestOSInfo(request *restful.Request, response *restful.Response) {
 	validate := func(vmi *v1.VirtualMachineInstance) *errors.StatusError {
 		if vmi == nil || vmi.Status.Phase != v1.Running {
-			return errors.NewConflict(v1.Resource("virtualmachineinstance"), vmi.Name, fmt.Errorf("VMI is not running"))
+			return errors.NewConflict(v1.Resource("virtualmachineinstance"), vmi.Name, fmt.Errorf(vmiNotRunning))
 		}
 		condManager := controller.NewVirtualMachineInstanceConditionManager()
 		if !condManager.HasCondition(vmi, v1.VirtualMachineInstanceAgentConnected) {
-			return errors.NewConflict(v1.Resource("virtualmachineinstance"), vmi.Name, fmt.Errorf("VMI does not have guest agent connected"))
+			return errors.NewConflict(v1.Resource("virtualmachineinstance"), vmi.Name, fmt.Errorf(vmiGuestAgentErr))
 		}
 		return nil
 	}
@@ -728,14 +849,14 @@ func (app *SubresourceAPIApp) GuestOSInfo(request *restful.Request, response *re
 
 	_, url, conn, err := app.prepareConnection(request, validate, getURL)
 	if err != nil {
-		log.Log.Errorf("Cannot prepare connection %s", err.Error())
+		log.Log.Errorf(prepConnectionErrFmt, err.Error())
 		response.WriteError(http.StatusInternalServerError, err)
 		return
 	}
 
 	resp, conErr := conn.Get(url, app.handlerTLSConfiguration)
 	if conErr != nil {
-		log.Log.Errorf("Cannot GET request %s", conErr.Error())
+		log.Log.Errorf(getRequestErrFmt, conErr.Error())
 		response.WriteError(http.StatusInternalServerError, conErr)
 		return
 	}
@@ -754,11 +875,11 @@ func (app *SubresourceAPIApp) GuestOSInfo(request *restful.Request, response *re
 func (app *SubresourceAPIApp) UserList(request *restful.Request, response *restful.Response) {
 	validate := func(vmi *v1.VirtualMachineInstance) *errors.StatusError {
 		if vmi == nil || vmi.Status.Phase != v1.Running {
-			return errors.NewConflict(v1.Resource("virtualmachineinstance"), vmi.Name, fmt.Errorf("VMI is not running"))
+			return errors.NewConflict(v1.Resource("virtualmachineinstance"), vmi.Name, fmt.Errorf(vmiNotRunning))
 		}
 		condManager := controller.NewVirtualMachineInstanceConditionManager()
 		if !condManager.HasCondition(vmi, v1.VirtualMachineInstanceAgentConnected) {
-			return errors.NewConflict(v1.Resource("virtualmachineinstance"), vmi.Name, fmt.Errorf("VMI does not have guest agent connected"))
+			return errors.NewConflict(v1.Resource("virtualmachineinstance"), vmi.Name, fmt.Errorf(vmiGuestAgentErr))
 		}
 		return nil
 	}
@@ -768,14 +889,14 @@ func (app *SubresourceAPIApp) UserList(request *restful.Request, response *restf
 
 	_, url, conn, err := app.prepareConnection(request, validate, getURL)
 	if err != nil {
-		log.Log.Errorf("Cannot prepare connection %s", err.Error())
+		log.Log.Errorf(prepConnectionErrFmt, err.Error())
 		response.WriteError(http.StatusInternalServerError, err)
 		return
 	}
 
 	resp, conErr := conn.Get(url, app.handlerTLSConfiguration)
 	if conErr != nil {
-		log.Log.Errorf("Cannot GET request %s", conErr.Error())
+		log.Log.Errorf(getRequestErrFmt, conErr.Error())
 		response.WriteError(http.StatusInternalServerError, conErr)
 		return
 	}
@@ -794,11 +915,11 @@ func (app *SubresourceAPIApp) UserList(request *restful.Request, response *restf
 func (app *SubresourceAPIApp) FilesystemList(request *restful.Request, response *restful.Response) {
 	validate := func(vmi *v1.VirtualMachineInstance) *errors.StatusError {
 		if vmi == nil || vmi.Status.Phase != v1.Running {
-			return errors.NewConflict(v1.Resource("virtualmachineinstance"), vmi.Name, fmt.Errorf("VMI is not running"))
+			return errors.NewConflict(v1.Resource("virtualmachineinstance"), vmi.Name, fmt.Errorf(vmiNotRunning))
 		}
 		condManager := controller.NewVirtualMachineInstanceConditionManager()
 		if !condManager.HasCondition(vmi, v1.VirtualMachineInstanceAgentConnected) {
-			return errors.NewConflict(v1.Resource("virtualmachineinstance"), vmi.Name, fmt.Errorf("VMI does not have guest agent connected"))
+			return errors.NewConflict(v1.Resource("virtualmachineinstance"), vmi.Name, fmt.Errorf(vmiGuestAgentErr))
 		}
 		return nil
 	}
@@ -808,14 +929,14 @@ func (app *SubresourceAPIApp) FilesystemList(request *restful.Request, response
 
 	_, url, conn, err := app.prepareConnection(request, validate, getURL)
 	if err != nil {
-		log.Log.Errorf("Cannot prepare connection %s", err.Error())
+		log.Log.Errorf(prepConnectionErrFmt, err.Error())
 		response.WriteError(http.StatusInternalServerError, err)
 		return
 	}
 
 	resp, conErr := conn.Get(url, app.handlerTLSConfiguration)
 	if conErr != nil {
-		log.Log.Errorf("Cannot GET request %s", conErr.Error())
+		log.Log.Errorf(getRequestErrFmt, conErr.Error())
 		response.WriteError(http.StatusInternalServerError, conErr)
 		return
 	}
@@ -995,7 +1116,7 @@ func (app *SubresourceAPIApp) addVolumeRequestHandler(request *restful.Request,
 		case io.EOF, nil:
 			break
 		default:
-			writeError(errors.NewBadRequest(fmt.Sprintf("Can not unmarshal Request body to struct, error: %s", err)), response)
+			writeError(errors.NewBadRequest(fmt.Sprintf(unmarshalRequestErrFmt, err)), response)
 			return
 		}
 	} else {
@@ -1018,6 +1139,11 @@ func (app *SubresourceAPIApp) addVolumeRequestHandler(request *restful.Request,
 	volumeRequest := v1.VirtualMachineVolumeRequest{
 		AddVolumeOptions: opts,
 	}
+	if opts.VolumeSource.DataVolume != nil {
+		opts.VolumeSource.DataVolume.Hotpluggable = true
+	} else if opts.VolumeSource.PersistentVolumeClaim != nil {
+		opts.VolumeSource.PersistentVolumeClaim.Hotpluggable = true
+	}
 
 	// inject into VMI if ephemeral, else set as a request on the VM to both make permanent and hotplug.
 	if ephemeral {
@@ -1052,7 +1178,7 @@ func (app *SubresourceAPIApp) removeVolumeRequestHandler(request *restful.Reques
 		case io.EOF, nil:
 			break
 		default:
-			writeError(errors.NewBadRequest(fmt.Sprintf("Can not unmarshal Request body to struct, error: %s",
+			writeError(errors.NewBadRequest(fmt.Sprintf(unmarshalRequestErrFmt,
 				err)), response)
 			return
 		}
@@ -1087,13 +1213,13 @@ func (app *SubresourceAPIApp) removeVolumeRequestHandler(request *restful.Reques
 }
 
 func (app *SubresourceAPIApp) vmiVolumePatch(name, namespace string, volumeRequest *v1.VirtualMachineVolumeRequest) *errors.StatusError {
-	vmi, statErr := app.fetchVirtualMachineInstance(name, namespace)
+	vmi, statErr := app.FetchVirtualMachineInstance(namespace, name)
 	if statErr != nil {
 		return statErr
 	}
 
 	if !vmi.IsRunning() {
-		return errors.NewConflict(v1.Resource("virtualmachineinstance"), name, fmt.Errorf("VMI is not running"))
+		return errors.NewConflict(v1.Resource("virtualmachineinstance"), name, fmt.Errorf(vmiNotRunning))
 	}
 
 	patch, err := generateVMIVolumeRequestPatch(vmi, volumeRequest)
@@ -1101,8 +1227,9 @@ func (app *SubresourceAPIApp) vmiVolumePatch(name, namespace string, volumeReque
 		return errors.NewConflict(v1.Resource("virtualmachineinstance"), name, err)
 	}
 
+	dryRunOption := app.getDryRunOption(volumeRequest)
 	log.Log.Object(vmi).V(4).Infof("Patching VMI: %s", patch)
-	if _, err := app.virtCli.VirtualMachineInstance(vmi.Namespace).Patch(vmi.Name, types.JSONPatchType, []byte(patch)); err != nil {
+	if _, err := app.virtCli.VirtualMachineInstance(vmi.Namespace).Patch(vmi.Name, types.JSONPatchType, []byte(patch), &k8smetav1.PatchOptions{DryRun: dryRunOption}); err != nil {
 		log.Log.Object(vmi).V(1).Errorf("unable to patch vmi: %v", err)
 		if errors.IsInvalid(err) {
 			if statErr, ok := err.(*errors.StatusError); ok {
@@ -1125,8 +1252,9 @@ func (app *SubresourceAPIApp) vmVolumePatchStatus(name, namespace string, volume
 		return errors.NewConflict(v1.Resource("virtualmachine"), name, err)
 	}
 
-	log.Log.Object(vm).V(4).Infof("Patching VM: %s", patch)
-	if err := app.statusUpdater.PatchStatus(vm, types.JSONPatchType, []byte(patch)); err != nil {
+	dryRunOption := app.getDryRunOption(volumeRequest)
+	log.Log.Object(vm).V(4).Infof(patchingVMFmt, patch)
+	if err := app.statusUpdater.PatchStatus(vm, types.JSONPatchType, []byte(patch), &k8smetav1.PatchOptions{DryRun: dryRunOption}); err != nil {
 		log.Log.Object(vm).V(1).Errorf("unable to patch vm status: %v", err)
 		if errors.IsInvalid(err) {
 			if statErr, ok := err.(*errors.StatusError); ok {
@@ -1138,6 +1266,16 @@ func (app *SubresourceAPIApp) vmVolumePatchStatus(name, namespace string, volume
 	return nil
 }
 
+func (app *SubresourceAPIApp) getDryRunOption(volumeRequest *v1.VirtualMachineVolumeRequest) []string {
+	var dryRunOption []string
+	if options := volumeRequest.AddVolumeOptions; options != nil && options.DryRun != nil && options.DryRun[0] == k8smetav1.DryRunAll {
+		dryRunOption = volumeRequest.AddVolumeOptions.DryRun
+	} else if options := volumeRequest.RemoveVolumeOptions; options != nil && options.DryRun != nil && options.DryRun[0] == k8smetav1.DryRunAll {
+		dryRunOption = volumeRequest.RemoveVolumeOptions.DryRun
+	}
+	return dryRunOption
+}
+
 // VMAddVolumeRequestHandler handles the subresource for hot plugging a volume and disk.
 func (app *SubresourceAPIApp) VMAddVolumeRequestHandler(request *restful.Request, response *restful.Response) {
 	app.addVolumeRequestHandler(request, response, false)
