@@ -26,11 +26,11 @@ import (
 
 	appsv1 "k8s.io/api/apps/v1"
 	corev1 "k8s.io/api/core/v1"
-	"k8s.io/api/policy/v1beta1"
+	policyv1 "k8s.io/api/policy/v1"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/apimachinery/pkg/util/intstr"
 
-	virtv1 "kubevirt.io/client-go/api/v1"
+	virtv1 "kubevirt.io/api/core/v1"
 	"kubevirt.io/kubevirt/pkg/virt-operator/resource/generate/rbac"
 	operatorutil "kubevirt.io/kubevirt/pkg/virt-operator/util"
 )
@@ -44,6 +44,8 @@ const (
 
 	kubevirtLabelKey              = "kubevirt.io"
 	kubernetesHostnameTopologyKey = "kubernetes.io/hostname"
+
+	portName = "--port"
 )
 
 func NewPrometheusService(namespace string) *corev1.Service {
@@ -112,7 +114,7 @@ func NewApiServerService(namespace string) *corev1.Service {
 	}
 }
 
-func newPodTemplateSpec(podName string, imageName string, repository string, version string, productName string, productVersion string, pullPolicy corev1.PullPolicy, podAffinity *corev1.Affinity, envVars *[]corev1.EnvVar) (*corev1.PodTemplateSpec, error) {
+func newPodTemplateSpec(podName string, imageName string, repository string, version string, productName string, productVersion string, productComponent string, pullPolicy corev1.PullPolicy, podAffinity *corev1.Affinity, envVars *[]corev1.EnvVar) (*corev1.PodTemplateSpec, error) {
 
 	version = AddVersionSeparatorPrefix(version)
 
@@ -122,9 +124,6 @@ func newPodTemplateSpec(podName string, imageName string, repository string, ver
 				virtv1.AppLabel:    podName,
 				prometheusLabelKey: prometheusLabelValue,
 			},
-			Annotations: map[string]string{
-				"scheduler.alpha.kubernetes.io/critical-pod": "",
-			},
 			Name: podName,
 		},
 		Spec: corev1.PodSpec{
@@ -149,6 +148,10 @@ func newPodTemplateSpec(podName string, imageName string, repository string, ver
 		podTemplateSpec.ObjectMeta.Labels[virtv1.AppPartOfLabel] = productName
 	}
 
+	if productComponent != "" {
+		podTemplateSpec.ObjectMeta.Labels[virtv1.AppComponentLabel] = productComponent
+	}
+
 	if envVars != nil && len(*envVars) != 0 {
 		podTemplateSpec.Spec.Containers[0].Env = *envVars
 	}
@@ -156,6 +159,23 @@ func newPodTemplateSpec(podName string, imageName string, repository string, ver
 	return podTemplateSpec, nil
 }
 
+func attachProfileVolume(spec *corev1.PodSpec) {
+
+	volume := corev1.Volume{
+		Name: "profile-data",
+		VolumeSource: corev1.VolumeSource{
+			EmptyDir: &corev1.EmptyDirVolumeSource{},
+		},
+	}
+	volumeMount := corev1.VolumeMount{
+		Name:      "profile-data",
+		MountPath: "/profile-data",
+	}
+	spec.Volumes = append(spec.Volumes, volume)
+	spec.Containers[0].VolumeMounts = append(spec.Containers[0].VolumeMounts, volumeMount)
+
+}
+
 func attachCertificateSecret(spec *corev1.PodSpec, secretName string, mountPath string) {
 	True := true
 	secretVolume := corev1.Volume{
@@ -177,9 +197,9 @@ func attachCertificateSecret(spec *corev1.PodSpec, secretName string, mountPath
 	spec.Containers[0].VolumeMounts = append(spec.Containers[0].VolumeMounts, secretVolumeMount)
 }
 
-func newBaseDeployment(deploymentName string, imageName string, namespace string, repository string, version string, productName string, productVersion string, pullPolicy corev1.PullPolicy, podAffinity *corev1.Affinity, envVars *[]corev1.EnvVar) (*appsv1.Deployment, error) {
+func newBaseDeployment(deploymentName string, imageName string, namespace string, repository string, version string, productName string, productVersion string, productComponent string, pullPolicy corev1.PullPolicy, podAffinity *corev1.Affinity, envVars *[]corev1.EnvVar) (*appsv1.Deployment, error) {
 
-	podTemplateSpec, err := newPodTemplateSpec(deploymentName, imageName, repository, version, productName, productVersion, pullPolicy, podAffinity, envVars)
+	podTemplateSpec, err := newPodTemplateSpec(deploymentName, imageName, repository, version, productName, productVersion, productComponent, pullPolicy, podAffinity, envVars)
 	if err != nil {
 		return nil, err
 	}
@@ -216,6 +236,10 @@ func newBaseDeployment(deploymentName string, imageName string, namespace string
 		deployment.ObjectMeta.Labels[virtv1.AppPartOfLabel] = productName
 	}
 
+	if productComponent != "" {
+		deployment.ObjectMeta.Labels[virtv1.AppComponentLabel] = productComponent
+	}
+
 	return deployment, nil
 }
 
@@ -243,18 +267,20 @@ func newPodAntiAffinity(key, topologyKey string, operator metav1.LabelSelectorOp
 	}
 }
 
-func NewApiServerDeployment(namespace string, repository string, imagePrefix string, version string, productName string, productVersion string, pullPolicy corev1.PullPolicy, verbosity string, extraEnv map[string]string) (*appsv1.Deployment, error) {
+func NewApiServerDeployment(namespace string, repository string, imagePrefix string, version string, productName string, productVersion string, productComponent string, pullPolicy corev1.PullPolicy, verbosity string, extraEnv map[string]string) (*appsv1.Deployment, error) {
 	podAntiAffinity := newPodAntiAffinity(kubevirtLabelKey, kubernetesHostnameTopologyKey, metav1.LabelSelectorOpIn, []string{VirtAPIName})
 	deploymentName := VirtAPIName
 	imageName := fmt.Sprintf("%s%s", imagePrefix, deploymentName)
 	env := operatorutil.NewEnvVarMap(extraEnv)
-	deployment, err := newBaseDeployment(deploymentName, imageName, namespace, repository, version, productName, productVersion, pullPolicy, podAntiAffinity, env)
+	deployment, err := newBaseDeployment(deploymentName, imageName, namespace, repository, version, productName, productVersion, productComponent, pullPolicy, podAntiAffinity, env)
 	if err != nil {
 		return nil, err
 	}
 
 	attachCertificateSecret(&deployment.Spec.Template.Spec, VirtApiCertSecretName, "/etc/virt-api/certificates")
 	attachCertificateSecret(&deployment.Spec.Template.Spec, VirtHandlerCertSecretName, "/etc/virt-handler/clientcertificates")
+	attachProfileVolume(&deployment.Spec.Template.Spec)
+
 	pod := &deployment.Spec.Template.Spec
 	pod.ServiceAccountName = rbac.ApiServiceAccountName
 	pod.SecurityContext = &corev1.PodSecurityContext{
@@ -264,7 +290,7 @@ func NewApiServerDeployment(namespace string, repository string, imagePrefix str
 	container := &deployment.Spec.Template.Spec.Containers[0]
 	container.Command = []string{
 		VirtAPIName,
-		"--port",
+		portName,
 		"8443",
 		"--console-server-port",
 		"8186",
@@ -309,12 +335,12 @@ func NewApiServerDeployment(namespace string, repository string, imagePrefix str
 	return deployment, nil
 }
 
-func NewControllerDeployment(namespace string, repository string, imagePrefix string, controllerVersion string, launcherVersion string, productName string, productVersion string, pullPolicy corev1.PullPolicy, verbosity string, extraEnv map[string]string) (*appsv1.Deployment, error) {
+func NewControllerDeployment(namespace string, repository string, imagePrefix string, controllerVersion string, launcherVersion string, productName string, productVersion string, productComponent string, pullPolicy corev1.PullPolicy, verbosity string, extraEnv map[string]string) (*appsv1.Deployment, error) {
 	podAntiAffinity := newPodAntiAffinity(kubevirtLabelKey, kubernetesHostnameTopologyKey, metav1.LabelSelectorOpIn, []string{VirtControllerName})
 	deploymentName := VirtControllerName
 	imageName := fmt.Sprintf("%s%s", imagePrefix, deploymentName)
 	env := operatorutil.NewEnvVarMap(extraEnv)
-	deployment, err := newBaseDeployment(deploymentName, imageName, namespace, repository, controllerVersion, productName, productVersion, pullPolicy, podAntiAffinity, env)
+	deployment, err := newBaseDeployment(deploymentName, imageName, namespace, repository, controllerVersion, productName, productVersion, productComponent, pullPolicy, podAntiAffinity, env)
 	if err != nil {
 		return nil, err
 	}
@@ -332,7 +358,7 @@ func NewControllerDeployment(namespace string, repository string, imagePrefix st
 		VirtControllerName,
 		"--launcher-image",
 		fmt.Sprintf("%s/%s%s%s", repository, imagePrefix, "virt-launcher", launcherVersion),
-		"--port",
+		portName,
 		"8443",
 		"-v",
 		verbosity,
@@ -375,6 +401,7 @@ func NewControllerDeployment(namespace string, repository string, imagePrefix st
 	}
 
 	attachCertificateSecret(pod, VirtControllerCertSecretName, "/etc/virt-controller/certificates")
+	attachProfileVolume(pod)
 
 	container.Resources = corev1.ResourceRequirements{
 		Requests: corev1.ResourceList{
@@ -424,9 +451,6 @@ func NewOperatorDeployment(namespace string, repository string, imagePrefix stri
 						virtv1.AppLabel:    VirtOperatorName,
 						prometheusLabelKey: prometheusLabelValue,
 					},
-					Annotations: map[string]string{
-						"scheduler.alpha.kubernetes.io/critical-pod": "",
-					},
 					Name: VirtOperatorName,
 				},
 				Spec: corev1.PodSpec{
@@ -441,7 +465,7 @@ func NewOperatorDeployment(namespace string, repository string, imagePrefix stri
 							ImagePullPolicy: pullPolicy,
 							Command: []string{
 								VirtOperatorName,
-								"--port",
+								portName,
 								"8443",
 								"-v",
 								verbosity,
@@ -537,6 +561,7 @@ func NewOperatorDeployment(namespace string, repository string, imagePrefix stri
 	}
 
 	attachCertificateSecret(&deployment.Spec.Template.Spec, VirtOperatorCertSecretName, "/etc/virt-operator/certificates")
+	attachProfileVolume(&deployment.Spec.Template.Spec)
 
 	return deployment, nil
 }
@@ -570,11 +595,14 @@ func AddVersionSeparatorPrefix(version string) string {
 	return version
 }
 
-func NewPodDisruptionBudgetForDeployment(deployment *appsv1.Deployment) *v1beta1.PodDisruptionBudget {
+func NewPodDisruptionBudgetForDeployment(deployment *appsv1.Deployment) *policyv1.PodDisruptionBudget {
 	pdbName := deployment.Name + "-pdb"
-	minAvailable := intstr.FromInt(int(1))
+	minAvailable := intstr.FromInt(1)
+	if deployment.Spec.Replicas != nil {
+		minAvailable = intstr.FromInt(int(*deployment.Spec.Replicas - 1))
+	}
 	selector := deployment.Spec.Selector.DeepCopy()
-	podDisruptionBudget := &v1beta1.PodDisruptionBudget{
+	podDisruptionBudget := &policyv1.PodDisruptionBudget{
 		ObjectMeta: metav1.ObjectMeta{
 			Namespace: deployment.Namespace,
 			Name:      pdbName,
@@ -582,7 +610,7 @@ func NewPodDisruptionBudgetForDeployment(deployment *appsv1.Deployment) *v1beta1
 				virtv1.AppLabel: pdbName,
 			},
 		},
-		Spec: v1beta1.PodDisruptionBudgetSpec{
+		Spec: policyv1.PodDisruptionBudgetSpec{
 			MinAvailable: &minAvailable,
 			Selector:     selector,
 		},
