@@ -65,7 +65,7 @@ import (
 	v1ext "k8s.io/apiextensions-apiserver/pkg/apis/apiextensions/v1"
 	extclient "k8s.io/apiextensions-apiserver/pkg/client/clientset/clientset"
 
-	v1 "kubevirt.io/client-go/api/v1"
+	v1 "kubevirt.io/api/core/v1"
 	"kubevirt.io/client-go/kubecli"
 	clusterutil "kubevirt.io/kubevirt/pkg/util/cluster"
 	"kubevirt.io/kubevirt/pkg/virt-controller/leaderelectionconfig"
@@ -152,7 +152,7 @@ var _ = Describe("[Serial][sig-compute]Infrastructure", func() {
 			Expect(slowDuration.Seconds()).To(BeNumerically(">", 2*fastDuration.Seconds()))
 		})
 
-		It("on the virt handler rate limiter should lead to delayed VMI running states", func() {
+		It("[QUARANTINE]on the virt handler rate limiter should lead to delayed VMI running states", func() {
 			By("first getting the basetime for a replicaset")
 			targetNode := util.GetAllSchedulableNodes(virtClient).Items[0]
 			vmi := libvmi.NewCirros(
@@ -258,7 +258,8 @@ var _ = Describe("[Serial][sig-compute]Infrastructure", func() {
 				secret, err := virtClient.CoreV1().Secrets(flags.KubeVirtInstallNamespace).Get(context.Background(), components.KubeVirtCASecretName, metav1.GetOptions{})
 				Expect(err).ToNot(HaveOccurred())
 				secret.Data = map[string][]byte{
-					"random": []byte("nonsense"),
+					"tls.crt": []byte(""),
+					"tls.key": []byte(""),
 				}
 
 				_, err = virtClient.CoreV1().Secrets(flags.KubeVirtInstallNamespace).Update(context.Background(), secret, metav1.UpdateOptions{})
@@ -344,7 +345,8 @@ var _ = Describe("[Serial][sig-compute]Infrastructure", func() {
 					return err
 				}
 				secret.Data = map[string][]byte{
-					"random": []byte("nonsense"),
+					"tls.crt": []byte(""),
+					"tls.key": []byte(""),
 				}
 				_, err = virtClient.CoreV1().Secrets(flags.KubeVirtInstallNamespace).Update(context.Background(), secret, metav1.UpdateOptions{})
 
@@ -1546,6 +1548,38 @@ var _ = Describe("[Serial][sig-compute]Infrastructure", func() {
 		})
 	})
 
+	Describe("cluster profiler for pprof data aggregation", func() {
+		BeforeEach(func() {
+			tests.BeforeTestCleanup()
+		})
+
+		Context("when ClusterProfiler feature gate", func() {
+			It("is disabled it should prevent subresource access", func() {
+				tests.DisableFeatureGate("ClusterProfiler")
+
+				err := virtClient.ClusterProfiler().Start()
+				Expect(err).ToNot(BeNil())
+
+				err = virtClient.ClusterProfiler().Stop()
+				Expect(err).ToNot(BeNil())
+
+				_, err = virtClient.ClusterProfiler().Dump(&v1.ClusterProfilerRequest{})
+				Expect(err).ToNot(BeNil())
+			})
+			It("[QUARANTINE] is enabled it should allow subresource access", func() {
+				tests.EnableFeatureGate("ClusterProfiler")
+
+				err := virtClient.ClusterProfiler().Start()
+				Expect(err).To(BeNil())
+
+				err = virtClient.ClusterProfiler().Stop()
+				Expect(err).To(BeNil())
+
+				_, err = virtClient.ClusterProfiler().Dump(&v1.ClusterProfilerRequest{})
+				Expect(err).To(BeNil())
+			})
+		})
+	})
 })
 
 func getLeader() string {
